﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AsyncAwait
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private int delay = 5000;
        private string result = "Bavaria ipsum dolor sit amet a Hoiwe muass, i bin a woschechta Bayer i sog ja nix, i red ja bloß großherzig obandln Leonhardifahrt Vergeltsgott. Pfenningguat Spuiratz sog i Schbozal.";

        public MainWindow()
        {
            InitializeComponent();
        }

        #region sync

        private void ButtonSync_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                txtResult.Text = string.Empty;
                string content = GetStringFromWeb();
                txtResult.Text = content;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), ex.Message);
            }
        }

        private string GetStringFromWeb()
        {
            Thread.Sleep(delay);

            if (this.Dispatcher.Invoke(() => txtInput.Text) == "EX")
            {
                throw new Exception();
            }

            return result;
        }

        #endregion

        #region async

        private async void ButtonAsync_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                txtResult.Text = string.Empty;
                string content = await GetStringFromWebAsync();
                txtResult.Text = content;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

        }

        private async Task<string> GetStringFromWebAsync()
        {
            await Task.Run(() => Thread.Sleep(delay));

            if (txtInput.Text == "EX")
            {
                throw new Exception();
            }

            return result;

        }

        #endregion

        #region tpl

        private void ButtonTPL_Click(object sender, RoutedEventArgs e)
        {
            txtResult.Text = string.Empty;
            Task<string> startTask = new Task<string>(() => GetStringFromWeb());
         
            startTask.ContinueWith(
                (lt) => txtResult.Text = lt.Result,
                CancellationToken.None,
                TaskContinuationOptions.OnlyOnRanToCompletion,
                TaskScheduler.FromCurrentSynchronizationContext());
           
            startTask.ContinueWith(
                (lt) => MessageBox.Show(lt.Exception.Message), 
                TaskContinuationOptions.OnlyOnFaulted);
          
            startTask.Start();
        }

        #endregion
    }
}
