﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading.Tasks.Dataflow;
using System.Runtime.CompilerServices;
using System.ComponentModel;
using System.Threading;
using System.Windows.Controls.Primitives;

namespace DataflowSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        int _value;
        public int Value { get { return _value; } set { _value = value; NotifyPropertyChanged(); } }

        ActionBlock<int> actionBlock;
        TransformBlock<string, int> stringToIntTransformBlock;
        TransformBlock<int[], int> intArrayToIntTransformBlock;
        BufferBlock<DateTime> bufferBlock;
        BatchBlock<int> batchBlock;
        BroadcastBlock<string> broadcastBlock;
        JoinBlock<int, int> joinBlock;

        IDisposable link;


        public MainWindow()
        {
            InitializeDataFlow();

            InitializeComponent();
            this.DataContext = this;

        }

        private void InitializeDataFlow()
        {

            actionBlock = new ActionBlock<int>(
                i =>
                {
                    Interlocked.Add(ref _value, i);
                    NotifyPropertyChanged("Value");
                });

            stringToIntTransformBlock = new TransformBlock<string, int>(s => int.Parse(s));
            stringToIntTransformBlock.LinkTo(actionBlock);


            batchBlock = new BatchBlock<int>(5);

            intArrayToIntTransformBlock = new TransformBlock<int[], int>(values => values.Sum());
            intArrayToIntTransformBlock.LinkTo(actionBlock);
            batchBlock.LinkTo(intArrayToIntTransformBlock);

            bufferBlock = new BufferBlock<DateTime>(new DataflowBlockOptions() { BoundedCapacity = DataflowBlockOptions.Unbounded });


            broadcastBlock = new BroadcastBlock<string>(i => i);
            broadcastBlock.LinkTo(stringToIntTransformBlock);

            joinBlock = new JoinBlock<int, int>();

            TransformBlock<Tuple<int, int>, int> tupletoIntTransformBlock =
                new TransformBlock<Tuple<int, int>, int>(t => t.Item1 * t.Item2);
            tupletoIntTransformBlock.LinkTo(actionBlock);
            joinBlock.LinkTo(tupletoIntTransformBlock);


            int res1;
            int res2;

            Parallel.Invoke(
                () => res1 = 3,
                () => res2 = 4);



        }

        private void Button_Action_Click(object sender, RoutedEventArgs e)
        {
            stringToIntTransformBlock.Post(txtAction.Text);
        }

        private void Button_Batch_Click(object sender, RoutedEventArgs e)
        {
            batchBlock.Post(int.Parse(txtAction.Text));
        }

        private void NotifyPropertyChanged([CallerMemberName]string propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void ToggleButton_Buffer_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox button = sender as CheckBox;
            if (button != null && button.IsChecked == true)
            {
                link = bufferBlock.LinkTo(new ActionBlock<DateTime>(dt => MessageBox.Show(dt.ToString())));
            }
            else
            {
                link.Dispose();
            }

        }

        private void Button_Buffer_Click(object sender, RoutedEventArgs e)
        {
            bufferBlock.Post(DateTime.Now);
        }

        private void Button_Broadcast_Click(object sender, RoutedEventArgs e)
        {
            broadcastBlock.Post(txtBroadCastSource.Text);
        }

        private void Button_BroadCast_Read(object sender, RoutedEventArgs e)
        {
            Button button = sender as Button;
            TextBlock textBlock = button.Tag as TextBlock;

            string value = string.Empty;
            if (!broadcastBlock.TryReceive(out value))
            {
                value = "no item found";
            }
            textBlock.Text = value;
        }

        private void Button_join_item1_post(object sender, RoutedEventArgs e)
        {
            joinBlock.Target1.Post(int.Parse(txtjoinItem1.Text));
        }

        private void Button_join_item2_post(object sender, RoutedEventArgs e)
        {
            joinBlock.Target2.Post(int.Parse(txtjoinItem2.Text));
        }
    }
}
